package net.wanji.common.utils.geo;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.toilelibre.libe.curl.Curl.curl;
import static org.toilelibre.libe.curl.Curl.$;

/**
 * GeoWebCache帮助类
 * @author guoliang.dong@wanji.com
 */
@Slf4j
public class GeoWebCacheUtils {

    private String geoUrl;
    private String geoUsername;
    private String geoPassword;

    private static GeoWebCacheUtils geoWebCacheUtils = null;

    public static GeoWebCacheUtils getGeoServerRESTManager(String url,String username,String password) {
        if (geoWebCacheUtils == null) {
            geoWebCacheUtils = new GeoWebCacheUtils(url,username,password);
        }
        return geoWebCacheUtils;
    }

    public GeoWebCacheUtils(String url,String username,String password){
        geoUrl=url;
        geoUsername=username;
        geoPassword=password;
    }


    /**
     * 新增工作空间
     *
     * @param workspace 工作空间名
     * @return boolean
     */
    public boolean addWorkspace(String workspace) {
        String cmd = "curl -u " + geoUsername + ":" + geoPassword + " -XPOST -H \"Content-type: text/xml\"\n" +
                "  -d \"<workspace><name>" + workspace + "</name></workspace>\"\n" +
                "  " + geoUrl + "/rest/workspaces";
        HttpResponse curl = curl(cmd);
        StatusLine statusLine = curl.getStatusLine();
        return statusLine.getStatusCode()==201;
    }


    /**
     * 创建外存储geotiff
     *
     * @param workspace
     * @param store
     * @return
     */
    public boolean addExternalGeotiffStore(String workspace ,String store,String fileurl){
        String cmd = "curl  -u " + geoUsername + ":" + geoPassword + " -XPUT -H \"Content-type: text/plain\"\n" +
                "  -d \"file://"+fileurl+"\"\n" +
                "  " + geoUrl + "/rest/workspaces/" + workspace + "/coveragestores/"+store+"/external.geotiff?configure=first&coverageName="+store;
        HttpResponse curl = curl(cmd);
        StatusLine statusLine = curl.getStatusLine();
        return statusLine.getStatusCode()==201;
    }

    /**
     * 获取geoWebCache中的图层
     *
     * @return Map
     */
    public Map<String, Object> getLayers() {
        Map<String, Object> map = new HashMap();
        String cmd = "curl -u " + geoUsername + ":" + geoPassword + " \"" + geoUrl + "/gwc/rest/layers\"";
        List<String> shp = new ArrayList<>();
        List<String> image = new ArrayList<>();
        HttpResponse curl = curl(cmd);
        HttpEntity entity = curl.getEntity();
        if (entity != null) {
            String result = null;
            try {
                result = EntityUtils.toString(entity, "UTF-8");
                JSONArray jsonArray = JSONArray.parseArray(result);
                for (Object o : jsonArray) {
                    String str = o.toString();
                    map.put(str,o);
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return map;
    }


    /**
     *
     *
     * @param layer
     * @param zoomStart
     * @param zoomStop
     * @return boolean
     */
    /**
     * 指定图层进行切片操作
     * @param layer 指定图层  shp:test
     * @param type  * seed (add tiles) * reseed (replace tiles) * truncate (remove tiles)
     * @param srs 坐标系srid
     * @param zoomStart 1 切片开始层级
     * @param zoomStop 15 切片结束层级
     * @return
     */
    public boolean slice(String layer, String type,int srs,int zoomStart, int zoomStop,String gridSetId) {
        int threadCount = 2;
        String cmd = "curl  -u " + geoUsername + ":" + geoPassword + " \"" + geoUrl + "/gwc/rest/seed/"+layer+".xml\""+ " -XPOST -H \"Content-type: text/xml\" -d '<seedRequest><name>" + layer +
                "</name><srs><number>"+srs+"</number></srs><zoomStart>" + zoomStart + "</zoomStart><zoomStop>" + zoomStop + "</zoomStop><format>image/png</format><type>"+type+"</type><threadCount>" + threadCount + "</threadCount><gridSetId>"+gridSetId+"</gridSetId></seedRequest>'  \""
                + geoUrl + "/gwc/rest/seed/" + layer + ".xml\"";
        HttpResponse curl = curl(cmd);
        StatusLine statusLine = curl.getStatusLine();
        return statusLine.getStatusCode()==200;
    }

    /**
     * 获取切片的情况
     *
     * @param layer 指定图层
     * @return Map
     */
    public Map getSliceType(String layer) {
        Map map = new HashMap();
        //返回所有图层切片情况   curl -u <user>:<password> -XGET http://localhost:8080/geoserver/gwc/rest/seed.json
        //返回指定图层的切片情况
        String cmd = "curl  -u " + geoUsername + ":" + geoPassword + " -XGET " + geoUrl + "/gwc/rest/seed/" + layer + ".json";
        HttpResponse curl = curl(cmd);
        StatusLine statusLine = curl.getStatusLine();
        if (statusLine.getStatusCode()==200) {
            HttpEntity entity = curl.getEntity();
            try {
                String result = EntityUtils.toString(entity, "UTF-8");
                JSONObject jsonArray = JSONObject.parseObject(result);
                map.put("res", jsonArray.getJSONArray("long-array-array"));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return map;
    }


    /**
     * 停止所有正在进行的切片任务
     *
     * @return boolean
     */
    public boolean stopAllSlice() {
        String cmd = "curl -u " + geoUsername + ":" + geoPassword + " -d \"kill_all=all\"  \"" + geoUrl + "/gwc/rest/seed\"";
        HttpResponse curl = curl(cmd);
        StatusLine statusLine = curl.getStatusLine();
        return statusLine.getStatusCode()==200;
    }

    /**
     * 停止指定图层的切片任务
     *
     * @return boolean
     */
    public boolean stopSliceByLayer(String layer) {
        String cmd = "curl -u " + geoUsername + ":" + geoPassword + " -d \"kill_all=all\"  \"" + geoUrl + "/gwc/rest/seed/" + layer + "\"";
        HttpResponse curl = curl(cmd);
        StatusLine statusLine = curl.getStatusLine();
        return statusLine.getStatusCode()==200;
    }



    public static void main(String[]args){
        GeoWebCacheUtils geoServerRESTManager=GeoWebCacheUtils.getGeoServerRESTManager("http://127.0.0.1:8080/geoserver","admin","geoserver");

        //System.out.println(geoServerRESTManager.addWorkspace("zhangwei"));

        //Map<String, Object> layers=geoServerRESTManager.getLayers();

        //System.out.println("图层数："+layers.size());
        //System.out.println(layers);

        System.out.println(geoServerRESTManager.slice("sf:result8","seed",4326,14,14,"My_EPSG:4326"));

        //System.out.println(geoServerRESTManager.stopSliceByLayer("gisc_3f0786e36794%3Apoi"));

        //System.out.println(geoServerRESTManager.getSliceType("gisc_3f0786e36794%3Apoi"));


        //System.out.println(geoServerRESTManager.getSliceType("gisc_3f0786e36794%3Apoi"));

        //System.out.println(geoServerRESTManager.stopAllSlice());

        //System.out.println(geoServerRESTManager.addExternalGeotiffStore("sf","result8","D:/ProgramData/GeoServer/data/sf/result8/result.tif"));

    }

}
